import damkjer.ocd.*; //for camera
import processing.opengl.*; //library for OpenGl
import proxml.*; //library for Vec3D
import noc.*; //library for Vecor3D 
import javax.media.opengl.*;
import ddf.minim.*;  //Audio library
import ddf.minim.analysis.*;
import processing.video.*;

Camera camera1;


XMLdata mydata;
boolean drawFlag = false;  // flag to draw visualization
boolean shift = false;
boolean drawLine = false;

AudioPlayer song; 
FFT fft;

int winSizeX = 800; //screen size for drawing
int winSizeY = 600;
int winSizeZ = 5000;

GraphGrid draw_g;
DrawData myParticles;
AnimateParticle aniP;

//OpenGL
PGraphicsOpenGL pgl;
GL gl;
int squareList;  //for OPENGL HERLPER

//image for Particle
PImage img;
PImage msk;
PImage shadow_msk;
PImage shadow_img;
PFont font, font2;

String MP3FILE = "data/secrete.mp3";
String xmlFile = "data/secrete.xml";


void setup(){

  size(1000, 700, OPENGL);
  background(0);
  hint( ENABLE_OPENGL_4X_SMOOTH );
 
  loadImg();
  font = loadFont("EuphemiaUCAS-14.vlw");
  font2 = loadFont("EuphemiaUCAS-14.vlw");
  textFont(font);
 
  mydata = new XMLdata();
  XMLInOut xml= new XMLInOut(this);

  camera1 = new Camera(this, 300, 300, 0, (width-winSizeX)/2+winSizeX/2, (height-winSizeY)/2+winSizeY/2, -(winSizeZ/2));

  Minim.start(this);
  
  //bufferSize 1024
  song = Minim.loadFile(MP3FILE,1024);
  fft = new FFT(song.mix.size(), 44100);
  
  draw_g = new GraphGrid();
  myParticles = new DrawData(mydata, img, shadow_img);

  // More OpenGL necessity.
  // setSwapInterval fixes a staggered screen refresh problem. Not sure if its
  // a Mac only problem or if there is a better way. All I know is it fixes the hiccups.
  pgl         = (PGraphicsOpenGL) g;
  gl          = pgl.gl;
  gl.setSwapInterval(1);

  initGL();

  //load XML file
  try{
    xml.loadElement(xmlFile); 
  }
  catch(Exception e){
    println("noXMLFile");
  }
  cameraInfo();
  
 aniP = new AnimateParticle(mydata,img, shadow_img );
 
}

void draw(){
  background(0);
  frameRate(24);
  gl.glDepthMask(false);
  gl.glEnable( GL.GL_BLEND );
  gl.glBlendFunc(GL.GL_SRC_ALPHA,GL.GL_ONE);

  camera1.feed();
  
   fft.forward(song.mix);
   float temp = 0;

  if(drawFlag){
    
     song.loop(); //play song
     
   for(int i=0; i< fft.specSize(); i++){
     temp = fft.getBand(i);  // gets the amplitude of the requested frequency (Hz) in the spectrum.
   }


    myParticles.setParticles(temp);
    aniP.drawData(song, temp);

 
  
  int index = aniP.currentSegmentIndex;
   
  if( mydata.stTime[index]+ mydata.durationSeg[index] >= mydata.durationOfSong){
      stop();
    }
    

  }

}



void xmlEvent(XMLElement element){
  mydata.parseElement (element);
  drawFlag = true;
}


void mouseDragged() {
  camera1.tumble(radians((mouseX - pmouseX)*PI/3), radians((mouseY - pmouseY)*PI/3));
}

void keyPressed(){
  //zoom in
  if((key == 'z') || ( key == 'Z')){
    camera1.dolly(-10);
  }
  //zoom out
  if( (key == 'x') || ( key == 'X')){
    camera1.dolly(10);
  } 
  
 if( (key == 's') || ( key == 'S')){
    println("stop");
    stop();
  }
  
 if( (key == 'l') || ( key == 'L')){
   if(drawLine != true){
   drawLine = true;
   }else{
     drawLine = false;
   }
  }
  
  //move up
  if(keyCode == UP){
    camera1.boom(10);
  }

  //move down
  if(keyCode == DOWN) {
    camera1.boom(-10);
  }

  //move left
  if(keyCode == LEFT){
    camera1.pan(-.1);
  }

  //move right 
  if(keyCode == RIGHT){
    camera1.pan(.1);
  }


}

//stop minim
void stop(){
  println("stop");

  song.close();
  Minim.stop();
  super.stop();  
  
}

void loadImg(){

  msk = loadImage("data/particleIMG.gif");
  img = new PImage(msk.width,msk.height);
  for (int i = 0; i < img.pixels.length; i++) img.pixels[i] = color(255);
  img.mask(msk);

  shadow_msk = loadImage("data/shadow.gif");
  shadow_img = new PImage(shadow_msk.width,shadow_msk.height);
  for (int i = 0; i < shadow_img.pixels.length; i++) shadow_img.pixels[i] = color(255);
  shadow_img.mask(shadow_msk);

}


//for Debugging Purpose
void cameraInfo(){
   float[] target = camera1.target();
  println("Target X: " + target[0]);
  println("Target Y: " + target[1]);
  println("Target Z: " + target[2]); 
}

void mouseClicked() {
  float[] position = camera1.position();
  println("Position X: " + position[0]);
  println("Position Y: " + position[1]);
  println("Position Z: " + position[2]);
 println("Field of View: " + camera1.fov());

 float[] up = camera1.up();
  println("Up X: " + up[0]);
  println("Up Y: " + up[1]);
  println("Up Z: " + up[2]);
}

//OPEN GL HELPERS
void initGL(){
  pgl.beginGL();
  squareList = gl.glGenLists(1);
  gl.glNewList(squareList, GL.GL_COMPILE);
  gl.glBegin(GL.GL_POLYGON);
  gl.glTexCoord2f(0, 0);    
  gl.glVertex2f(-.5, -.5);
  gl.glTexCoord2f(1, 0);    
  gl.glVertex2f( .5, -.5);
  gl.glTexCoord2f(1, 1);    
  gl.glVertex2f( .5,  .5);
  gl.glTexCoord2f(0, 1);    
  gl.glVertex2f(-.5,  .5);
  gl.glEnd();
  gl.glEndList();
  pgl.endGL();
}
